module Sluggable
  extend ActiveSupport::Concern

  included do 
    before_save :generate_slug!
    class_attribute :slug_column
  end

  def to_param
    self.slug
  end

  def generate_slug!
    new_slug = to_slug(self.send(self.class.slug_column.to_sym))

    # if a obj with this slug exists, uniquify it
    obj = self.class.find_by slug: new_slug

    count = 2
    while obj && obj != self
      new_slug = append_suffix(new_slug, count)
      obj = self.class.find_by slug: new_slug
      count += 1
    end

    self.slug = new_slug
  end

  def append_suffix(str, count)
    if str.split('-').last.to_i != 0
      return str.split("-").slice(0...-1).join("-") + "-" + count.to_s
    else
      return str + "-" + count.to_s
    end
  end

  def to_slug(name)
    str = name.strip.downcase
    str.gsub!(/\s*[^a-z0-9]\s*/, "-")
    str.gsub!(/-+/, "-")
    return str
  end

  module ClassMethods
    def sluggable_column(col_name)
      self.slug_column = col_name
    end
  end
end